/* query_log.c - Log queries in a way we can trace
   Copyright (C) 2003 David G. Andersen

This file is part of the pdnsd package hacked for MIT.

pdnsd is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

pdnsd is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with pdsnd; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <config.h>
#include <sys/types.h>
#include <stdio.h>
#include <stdarg.h>
#include <syslog.h>
#include <pthread.h>
#include <signal.h>
#include <string.h>
#include <errno.h>
#include "error.h"
#include "helpers.h"
#include "conff.h"

#if !defined(lint) && !defined(NO_RCSIDS)
static char rcsid[]="$Id: query_log.c,v 1.2 2004/07/06 20:29:01 dga Exp $";
#endif

pthread_mutex_t queryloglock = PTHREAD_MUTEX_INITIALIZER;
time_t query_open_time = 0;
FILE *querylog = NULL;

int lock_query() {
    return softlock_mutex(&queryloglock);
}

int unlock_query() {
    return pthread_mutex_unlock(&queryloglock);
}

/* This function assumes the query lock is held. */
char *query_timestamp() {
    static char timestamp[32];
    struct timeval tv;
    gettimeofday(&tv, NULL);
    sprintf(timestamp, "%u.%6.6u", tv.tv_sec, tv.tv_usec);
    return timestamp;
}

void open_querylog()
{
    char logname[256];

    query_open_time = time(NULL);
    sprintf(logname, "%s.%u", global.query_log_file, query_open_time);

    if (!querylog) {
	querylog = fopen(logname, "a");
	if (!querylog) {
	    log_error("could not open querylog %s: %s\n",
		      logname, strerror(errno));
	    return;
	}
    }
}

void close_querylog(int do_lock)
{
    int ul = 0;
    char oldname[256], newname[256];
    if (!querylog) return;
    if (do_lock)
	ul = softlock_mutex(&queryloglock);
    sprintf(oldname, "%s.%u", global.query_log_file, query_open_time);
    sprintf(newname, "%s.%u.%u", global.query_log_file, query_open_time,
	    time(NULL));
    fclose(querylog);
    rename(oldname, newname);
    if (ul) pthread_mutex_unlock(&queryloglock);
}

void reopen_querylog()
{
    int ul = 0;
    ul = softlock_mutex(&queryloglock);
    if (querylog) close_querylog(0);
    open_querylog();
    if (ul) pthread_mutex_unlock(&queryloglock);
}

void log_query(int do_lock, char *s,...)
{
	int ul=0;
	va_list va;
	va_start(va,s);
	
	if (!global.query_log_file) {
	    return;
	}

	if (do_lock) {
	    ul=softlock_mutex(&queryloglock);
	}

	if (!querylog) {
	    open_querylog();
	    if (!querylog) {
		if (ul)
		    pthread_mutex_unlock(&queryloglock);
		return;
	    }
	}

	if (do_lock) {
	    fprintf(querylog, "%s ", query_timestamp());
	}
	vfprintf(querylog, s, va);
	if (ul)
		pthread_mutex_unlock(&queryloglock);
	fflush(querylog);
	va_end(va);
}

